module InfernoRegisterFile (
    input clk,
    input reset,

    input [15:0] b_ac,
    input [2:0] ra,
    input [2:0] rb,
    input [2:0] rd,
    input [4:0] ctl,
    output wire [15:0] q_a,
    output wire [15:0] q_b
);

    wire [2:0] ra_out;
    wire [2:0] q_rd;
    wire [2:0] q_ra;
    wire [2:0] q_rb;
    
    wire [15:0] q_r0;
    wire [15:0] q_r1;
    wire [15:0] q_r2;
    wire [15:0] q_r3;
    wire [15:0] q_r4;
    wire [15:0] q_r5;
    wire [15:0] q_r6;
    wire [15:0] q_r7;

    wire ce0;
    wire ce1;
    wire ce2;
    wire ce3;
    wire ce4;
    wire ce5;
    wire ce6;
    wire ce7;

    wire [15:0] d0;
    wire [15:0] d1;
    wire [15:0] d2;
    wire [15:0] d3;
    wire [15:0] d4;
    wire [15:0] d5;
    wire [15:0] d6;
    wire [15:0] d7;

    mux16_8 ra_mux (
        .sel(q_ra),
        .a(q_r0),
        .b(q_r1),
        .c(q_r2),
        .d(q_r3),
        .e(q_r4),
        .f(q_r5),
        .g(q_r6),
        .h(q_r7),
        .out(q_a)
    );

    mux16_8 rb_mux (
        .sel(q_rb),
        .a(q_r0),
        .b(q_r1),
        .c(q_r2),
        .d(q_r3),
        .e(q_r4),
        .f(q_r5),
        .g(q_r6),
        .h(q_r7),
        .out(q_b)
    );

    mux3_2 q_ra_mux (
        .sel(ctl[1]),
        .a(ra_out),
        .b(q_rd),
        .out(q_ra)
    );

    reg16_a r0 (.clk(clk), .reset(reset), .d(d0), .q(q_r0), .ce(ce0));
    reg16_a r1 (.clk(clk), .reset(reset), .d(d1), .q(q_r1), .ce(ce1));
    reg16_a r2 (.clk(clk), .reset(reset), .d(d2), .q(q_r2), .ce(ce2));
    reg16_a r3 (.clk(clk), .reset(reset), .d(d3), .q(q_r3), .ce(ce3));
    reg16_a r4 (.clk(clk), .reset(reset), .d(d4), .q(q_r4), .ce(ce4));
    reg16_a r5 (.clk(clk), .reset(reset), .d(d5), .q(q_r5), .ce(ce5));
    reg16_a r6 (.clk(clk), .reset(reset), .d(d6), .q(q_r6), .ce(ce6));
    reg16_a r7 (.clk(clk), .reset(reset), .d(d7), .q(q_r7), .ce(ce7));

    demux16_8 d_mux (
        .sel(q_rd),
        .in(b_ac),
        .a(d0),
        .b(d1),
        .c(d2),
        .d(d3),
        .e(d4),
        .f(d5),
        .g(d6),
        .h(d7)
    );

    demux1_8 ce_mux (
        .sel(q_rd),
        .in(ctl[3]),
        .a(ce0),
        .b(ce1),
        .c(ce2),
        .d(ce3),
        .e(ce4),
        .f(ce5),
        .g(ce6),
        .h(ce7)
    );

    reg3_a rd_reg (.clk(clk), .reset(reset), .d(rd), .q(q_rd), .ce(ctl[4]));
    reg3_a ra_reg (.clk(clk), .reset(reset), .d(ra), .q(ra_out), .ce(ctl[4]));
    reg3_a rb_reg (.clk(clk), .reset(reset), .d(rb), .q(q_rb), .ce(ctl[4]));

endmodule

module InfernoAccumulator (
    input clk,
    input reset,

    input [15:0] b,

    input [2:0] ra,
    input [2:0] rb,
    input [2:0] rd,

    input [24:0] ctl,
    output eq,
    output [15:0] q
);

    wire [15:0] mux_a;
    wire [15:0] mux_b;


    InfernoAlu alu (.clk(clk), .a(mux_a), .b(mux_b), .f(q), .eq(eq), .ci(1'b0), .ctl(ctl[24:21]));

    InfernoRegisterFile register (
        .clk(clk),
        .reset(reset),
        .b_ac(b),
        .q_a(mux_a),
        .q_b(mux_b),
        .ra(ra),
        .rb(rb),
        .rd(rd),
        .ctl(ctl[20:16])
    );

endmodule