module InfernoController (
    input clk,
    input reset,
    input start,
    input eq,
    output reg [24:0] ctl,
    input [6:0] opc
);

    reg [2:0] state;
    reg [2:0] next_state;

    parameter s0 = 3'b000;
    parameter s1 = 3'b001;
    parameter s2 = 3'b010;
    parameter s3 = 3'b011;
    parameter s4 = 3'b100;
    parameter s5 = 3'b101;
    parameter s6 = 3'b110;

    always @(posedge clk) begin
        state <= next_state;
    end

    always @(state or reset or start or opc or eq) begin
        if (reset)
            next_state <= s0;
        else
            case (state)
                s0:
                    begin
                        if (start) begin
                            ctl <= 25'b0000000000000001001000110; // load pc with ar_reg
                            next_state <= s1;
                        end else begin
                            ctl <= 25'b0000000000000001000000000;
                            next_state <= s0;
                        end
                    end
                s1:
                    begin
                        ctl <= 25'b0000000100000000010000100; // inc pc and input mr_reg
                        next_state <= s2;
                    end
                s2:
                    begin
                        ctl <= 25'b0000000100001000001100000; // pc input ar_reg and load ir_reg
                        next_state <= s3;
                    end
                s3:
                    begin
                        if (opc[1:0] == 2'b00) begin
                            ctl <= 25'b0000100000000000000000000;
                            next_state <= s4;
                        end else if (opc[1:0] == 2'b10) begin
                            ctl <= 25'b0000100100000000010000100;
                            next_state <= s4;
                        end else if (opc[1:0] == 2'b11) begin
                            ctl <= 25'b0000000100000000010000100;
                            next_state <= s4;
                        end
                    end
                s4:
                    if (opc == 7'b1111000) begin // hlt
                        ctl <= 25'b0;
                        next_state <= s0;
                    end else if (opc == 7'b0000000) begin // nop
                        ctl <= 25'b0000000000000000001100000;
                        next_state <= s1;
                    end else if (opc == 7'b1000000) begin // add
                        ctl <= 25'b0001010000000000001100000;
                        next_state <= s1;
                    end else if (opc == 7'b1000100) begin // sub
                        ctl <= 25'b0010010000000000001100000;
                        next_state <= s1;
                    end else if (opc == 7'b1001000) begin // and
                        ctl <= 25'b0011010000000000001100000;
                        next_state <= s1;
                    end else if (opc == 7'b1001100) begin // or
                        ctl <= 25'b0100010000000000001100000;
                        next_state <= s1;
                    end else if (opc == 7'b1010000) begin // xor
                        ctl <= 25'b0101010000000000001100000;
                        next_state <= s1;
                    end else if (opc == 7'b0101100) begin // nand
                        ctl <= 25'b1110010000000000001100000;
                        next_state <= s1;
                    end else if (opc == 7'b0110000) begin // dec
                        ctl <= 25'b0111010100000000001100000;
                        next_state <= s1;
                    end else if (opc == 7'b0110100) begin // inc
                        ctl <= 25'b0110010100000000001100000;
                        next_state <= s1;
                    end else if (opc == 7'b0100100) begin // shr
                        ctl <= 25'b1001010000000000001100000;
                        next_state <= s1;
                    end else if (opc == 7'b0101000) begin // shl
                        ctl <= 25'b1000010000000000001100000;
                        next_state <= s1;
                    end else if (opc == 7'b1100000) begin // cpl
                        ctl <= 25'b1011010100000000001100000;
                        next_state <= s1;
                    end else if (opc == 7'b1011100) begin // clr
                        ctl <= 25'b1111010100000000001100000;
                        next_state <= s1;
                    end else if (opc == 7'b0000110) begin // load
                        ctl <= 25'b0000000100000001101000000;
                        next_state <= s5;
                    end else if (opc == 7'b0001010) begin // store
                        ctl <= 25'b0000000000000001101000000;
                        next_state <= s5;
                    end else if (opc == 7'b1010100) begin // mov
                        ctl <= 25'b1010010000000000001100000;
                        next_state <= s1;
                    end else if (opc == 7'b1100100) begin // inx
                        ctl <= 25'b0000010101010001001100000;
                        next_state <= s1;
                    end else if (opc == 7'b1101000) begin // outx
                        ctl <= 25'b0000000100000100101100000;
                        next_state <= s1;
                    
                    // --- jumps ---
                    
                    end else if (opc == 7'b0010011) begin // jmp
                        ctl <= 25'b0000000000000001110000110;
                        next_state <= s5;
                    end else if (opc == 7'b0011011 && eq == 1'b0) begin // jz + nz
                        ctl <= 25'b0000000000000000001100000;
                        next_state <= s1;
                    end else if (opc == 7'b0011011 && eq == 1'b1) begin // jz + z
                        ctl <= 25'b0000000000000001110000110;
                        next_state <= s5;

                    end else if (opc == 7'b0000100) begin // load (addr in rb)
                        ctl <= 25'b0000000000000000101000000;
                        next_state <= s5;
                    end else if (opc == 7'b0001000) begin // store (addr in rb)
                        ctl <= 25'b0000000000000000101000000;
                        next_state <= s5;
                    end
                
                s5:
                    if (opc == 7'b0000110) begin // load
                        ctl <= 25'b0000000100000000010000000;
                        next_state <= s6;
                    end else if (opc == 7'b0001010) begin // store
                        ctl <= 25'b0000000100100000100000000;
                        next_state <= s6;
                    end else if (opc == 7'b0011011) begin // jz
                        ctl <= 25'b0000000000000000001100000;
                        next_state <= s1;
                    end else if (opc == 7'b0010011) begin // jump
                        ctl <= 25'b0000000000000000001100000;
                        next_state <= s1;

                    end else if (opc == 7'b0000100) begin // load (addr in rb)
                        ctl <= 25'b0000000100000000010000000;
                        next_state <= s6;
                    end else if (opc == 7'b0001000) begin // store (addr in rb)
                        ctl <= 25'b0000000100100000100000000;
                        next_state <= s6;
                    end
                
                s6:
                    if (opc == 7'b0000110) begin // load
                        ctl <= 25'b0000010100010001101100000;
                        next_state <= s1;
                    end else if (opc == 7'b0001010) begin // store
                        ctl <= 25'b0000000000000000001100000;
                        next_state <= s1;
                    end else if (opc == 7'b0000100) begin // load (addr in rb)
                        ctl <= 25'b0000010100010001101100000;
                        next_state <= s1;
                    end else if (opc == 7'b0001000) begin // store (addr in rb)
                        ctl <= 25'b0000000000000000001100000;
                        next_state <= s1;
                    end

            endcase
    end

endmodule