module InfernoExecutionUnit (
    input clk,
    input reset,

    input [15:0] ipx_d,
    output wire [15:0] opx_q,

    input [15:0] mr_d,
    input [24:0] ctl,

    output wire eq,
    output wire [15:0] ar_q,
    output wire [6:0] ir_q,
    output wire [15:0] sysbus
);

    wire [15:0] a_q;
    wire [15:0] ar_d;
    wire [15:0] mux_bus;
    wire tbuf_en;
    wire [15:0] pc_q;
    wire [15:0] akku_b;
    wire [15:0] mr_q;
    wire [15:0] ipx_q;

    InfernoProgrammCounter pc (
        .clk(clk),
        .reset(reset),
        .d(sysbus),
        .q(pc_q),
        .ctl(ctl[2:1])
    );

    mux16_2 akku_mux (
        .a(a_q),
        .b(sysbus),
        .out(akku_b),
        .sel(ctl[13])
    );

    mux16_2 ar_mux (
        .a(sysbus),
        .b(pc_q),
        .out(ar_d),
        .sel(ctl[5])
    );

    InfernoAccumulator accumulator (
        .clk(clk),
        .reset(reset),
        .b(akku_b),
        .rd(mr_q[8:6]),
        .ra(mr_q[5:3]),
        .rb(mr_q[2:0]),
        .q(a_q),
        .ctl(ctl),
        .eq(eq)
    );

    reg16_a ar_reg (.clk(clk), .reset(reset), .d(ar_d), .q(ar_q), .ce(ctl[6]));
    reg16_a mr_reg (.clk(clk), .reset(reset), .d(mr_d), .q(mr_q), .ce(ctl[7]));
    reg16_a ipx_reg (.clk(clk), .reset(reset), .d(ipx_d), .q(ipx_q), .ce(ctl[9]));
    reg16_a opx_reg (.clk(clk), .reset(reset), .d(sysbus), .q(opx_q), .ce(ctl[11]));

    reg7_a ir_reg (.clk(clk), .reset(reset), .d(mr_q[15:9]), .q(ir_q), .ce(ctl[12]));

    tbuf16 tbuf (.d(mux_bus), .q(sysbus), .en(tbuf_en));

    assign tbuf_en = ctl[8] | ctl[9];

    mux16_4 bus_mux (
        .a(16'b0),
        .b(a_q),
        .c(ipx_q),
        .d(mr_q),
        .out(mux_bus),
        .sel(ctl[9:8])
    );
endmodule