module InfernoMMIO (
    input clk,
    input reset,

    output reg [15:0] do,
    input [15:0] di,

    input [15:0] addr,

    input we,

    output is_mmio,

    input rx,
    output tx
);

    wire [15:0] uart_clk_scaler_do;
    wire [15:0] uart_rx_do;
    wire [15:0] uart_tx_do;

    InfernoUart uart (
        .clk(clk),
        .reset(reset),
        .di(di),
        .do_tx(uart_tx_do),
        .do_rx(uart_rx_do),
        .do_scaler(uart_clk_scaler_do),
        .we_clk_scaler(we && (addr == 16'hfff0)),
        .we_tx(we && (addr == 16'hfff2)),
        .rx(rx),
        .tx(tx)
    );



    // do mux
    always @(posedge clk) begin
        case (addr)
            16'hfff0: do <= uart_clk_scaler_do; 
            16'hfff1: do <= uart_rx_do; 
            16'hfff2: do <= uart_tx_do;
            default: do <= 16'b0; 
        endcase        
    end

    assign is_mmio = (addr == 16'hfff0) | (addr == 16'hfff1) | (addr == 16'hfff2);
endmodule