module Inferno (
    input clk,
    input reset,

    input [15:0] ipx_d,
    output [15:0] opx_q,

    input start,

    input rx,
    output tx,

    input p_in,
    input p_clk,
    input p_we
);

    wire [15:0] addr_in;
    wire [15:0] ar_q;
    wire [15:0] sys_in;
    
    wire [15:0] da_1;

    wire [15:0] do_ram;
    wire [15:0] do_mmio;
    
    wire wr_in;
    wire clk_cpu1;
    wire clk_cpu2;


    wire wr_ram;
    wire wr_mmio;

    wire is_mmio;

    InfernoCore core (
        .clk(clk_cpu2),
        .reset(reset),
        .mr_d(da_1),
        .ipx_d(ipx_d),
        .ar_q(addr_in),
        .start(start),
        .opx_q(opx_q),
        .sysbus(sys_in),
        .wr_ram(wr_in)
    );

    mux16_2 da_1_mux (
        .sel(is_mmio),
        .a(do_ram),
        .b(do_mmio),
        .out(da_1)
    );

    demux1_2 wr_in_demux (
        .sel(is_mmio),
        .a(wr_ram),
        .b(wr_mmio),
        .in(wr_in)
    );

    InfernoMemory memory (
        .clk(clk_cpu1),
        .addr(addr_in),
        .di(sys_in),
        .do(do_ram),
        .we(wr_ram),
        .p_in(p_in),
        .p_clk(p_clk),
        .p_we(p_we)
    );

    InfernoMMIO mmio (
        .clk(clk_cpu1),
        .reset(reset),
        .addr(addr_in),
        .di(sys_in),
        .do(do_mmio),
        .we(wr_mmio),
        .is_mmio(is_mmio),
        .rx(rx),
        .tx(tx)
    );

    clk_mod clk_mod (
        .clk(clk),
        .reset(reset),
        .clk_1(clk_cpu1),
        .clk_2(clk_cpu2)
    );

endmodule