
module InfernoUartRX (
    input clk,
    input reset,

    input [15:0] scaler,

    output [15:0] do,

    input rx
);

    reg [7:0] buffer;

    assign do[7:0] = buffer;
    assign do[15:8] = 8'b0;

    // parameter CLK_FREQ = 27000000;
    // parameter UART_FREQ = 115200;

    localparam STATE_IDLE   = 2'b00;
    localparam STATE_START  = 2'b01;
    localparam STATE_DATA   = 2'b10;
    localparam STATE_STOP   = 2'b11;

    reg [7:0] data = 8'h00;
    reg [2:0] bitpos = 3'h0;
    reg [1:0] state;

    // localparam RX_CLK_MAX = (CLK_FREQ / UART_FREQ) - 1;
    // reg[$clog2(RX_CLK_MAX + 1) + 1:0] rx_clkcnt;
    reg[15:0] rx_clkcnt;
    wire rx_clk = (rx_clkcnt == 0);

    always @(posedge clk or posedge reset) begin
        if (reset) begin
            rx_clkcnt <= 0;
        end else begin
            if (rx_clkcnt >= scaler) begin
                rx_clkcnt <= 0;
            end else begin
                rx_clkcnt <= rx_clkcnt + 1;
            end
        end
    end

    always @(posedge clk or posedge reset) begin
        if (reset) begin
            buffer <= 8'h00;
            state <= STATE_IDLE;
        end else begin
            case (state)
                STATE_IDLE:
                    begin
                        if (rx == 1'b0) begin
                            state <= STATE_START;
                            bitpos <= 3'h0;
                        end
                    end
                STATE_START:
                    begin
                        if (rx_clk) begin
                            if (rx == 1'b0) begin
                                state <= STATE_DATA;
                            end else begin
                                state <= STATE_IDLE;
                            end
                        end
                    end
                STATE_DATA:
                    begin
                        if (rx_clk) begin
                            if (bitpos == 3'h7)
                                state <= STATE_STOP;
                            else
                                bitpos <= bitpos + 3'h1;
                            data[bitpos] <= rx;
                        end
                    end
                STATE_STOP:
                    begin
                        if (rx_clk) begin
                            if (rx == 1'b1) begin
                                buffer <= data;
                                state <= STATE_IDLE;
                            end
                        end
                    end
            endcase
        end
    end
endmodule


module InfernoUartTX (
    input clk,
    input reset,

    input [15:0] scaler,

    input [15:0] di,
    output [15:0] do,

    input we,

    output reg tx
);

    // parameter clk_freq = (27000000 / 2);
    // parameter uart_freq = 115200;

    localparam STATE_IDLE	= 2'b00;
    localparam STATE_START	= 2'b01;
    localparam STATE_DATA	= 2'b10;
    localparam STATE_STOP	= 2'b11;


    reg [7:0] data = 8'h00;
    reg [2:0] bitpos = 3'h0;
    reg [1:0] state;

    assign do[1:0] = state;
    assign do[15:2] = 14'b0;


    // localparam TX_CLK_MAX = (clk_freq / uart_freq) - 1;
    // reg[$clog2(TX_CLK_MAX + 1) + 1:0] tx_clkcnt;
    reg[15:0] tx_clkcnt;
    wire tx_clk = (tx_clkcnt == 0);

    always @(posedge clk or posedge reset) begin
        if (reset) begin
            tx_clkcnt <= 0;
        end else begin
            if (tx_clkcnt >= scaler) begin
                tx_clkcnt <= 0;
            end else begin
                tx_clkcnt <= tx_clkcnt + 1;
            end
        end
    end

    always @(posedge clk or posedge reset) begin
        if (reset) begin
            tx <= 1'b1;
            state <= STATE_IDLE;
        end else begin
            case (state)
                STATE_IDLE:
                    begin
                        if (we) begin
                            state <= STATE_START;
                            data <= di[7:0];
                            bitpos <= 3'h0;
                        end
                    end
                STATE_START:
                    begin
                        if (tx_clk) begin
                            tx <= 1'b0;
                            state <= STATE_DATA;
                        end
                    end
                STATE_DATA:
                    begin
                        if (tx_clk) begin
                            if (bitpos == 3'h7)
                                state <= STATE_STOP;
                            else
                                bitpos <= bitpos + 3'h1;
                            tx <= data[bitpos];
                        end
                    end
                STATE_STOP:
                    begin
                        if (tx_clk) begin
                            tx <= 1'b1;
                            state <= STATE_IDLE;
                        end
                    end
            endcase
        end
    end
endmodule

module InfernoUart (
    input clk,
    input reset,

    input [15:0] di,

    output [15:0] do_tx,
    output [15:0] do_rx,
    output [15:0] do_scaler,

    input we_clk_scaler,
    input we_tx,

    input rx,
    output tx
);

    reg16_a uart_clk_scaler (
        .clk(clk),
        .reset(reset),
        .ce(we_clk_scaler),
        .d(di),
        .q(do_scaler)
    );

    InfernoUartRX uart_rx (
        .clk(clk),
        .reset(reset),
        .scaler(do_scaler),
        .do(do_rx),
        .rx(rx)
    );

    InfernoUartTX uart_tx (
        .clk(clk),
        .reset(reset),
        .scaler(do_scaler),
        .di(di),
        .do(do_tx),
        .we(we_tx),
        .tx(tx)
    );
endmodule